"""
把以字节为单位的文件大小转换为对人类友好的方式
1024 B --> 1KiB
1024 * 1024 * 3 --> 3MiB
......
"""

def fileformat(size,base=1000):
    """把文件大小从字节转化成对人类友好的形式
    >>> print(fileformat(1024))
    1.0 KiB
    >>> print(fileformat(1024 * 1024))
    1.0 MiB
    >>> print(fileformat(0))
    0.0 B
    >>> print(fileformat(6))
    6.0 B
    """
    if base not in (1000,1024):
        # 为么以 1000 进位，要么以 1024 进位
        raise ValueError("the base argument must be 1000 or 1024 .")

    size = float(size)
    if base == 1024:
        suffix = ['KiB','MiB','GiB','TiB','PiB','EiB','ZiB','YiB']
    else:
        suffix = ['KB','MB','GB','TB','PB','EB','ZB','YB']
    
    if size < base:
        return f"{size} B"
    else:
        for i,suf in enumerate(suffix):
            unit = base ** (i + 2)
            if size < unit:
                return "{0:.1f} {1}".format(size * base /unit,suf)
        return "{0:.1f} {1}".format(size * base /unit,suf)


if __name__ == "__main__":
    import doctest
    doctest.testmod()



#-*- coding:utf8 -*-

"""
为show global status 中的statu提供一个与之对应的类，以方便使用
"""
from .base import StatuBase,ShowSlave

__all__=['RplSemiSyncMasterClients','RplSemiSyncMasterStatus','RplSemiSyncMasterNoTx',
         'RplSemiSyncMasterYesTx','RplSemiSyncSlaveStatus','SlaveIORunning','SlaveSQLRunning',
         'SecondsBehindMaster']


class RplSemiSyncMasterClients(StatuBase):
    """
    The number of semisynchronous slaves
    """
    statu_name="Rpl_semi_sync_master_clients"

class RplSemiSyncMasterStatus(StatuBase):
    """
    The value is 1 if the plugin has been enabled  It is 0 if the plugin is not enabled
    """
    statu_name="Rpl_semi_sync_master_status"

class RplSemiSyncMasterNoTx(StatuBase):
    """
    The number of commits that were not acknowledged successfully by a slave
    """
    statu_name="Rpl_semi_sync_master_no_tx"

class RplSemiSyncMasterYesTx(StatuBase):
    """
    The number of commits that were acknowledged successfully by a slave
    """
    statu_name="Rpl_semi_sync_master_yes_tx"

class RplSemiSyncSlaveStatus(StatuBase):
    """
    Whether semisynchronous replication currently is operational on the slave. This is 1 if the plugin has been enabled and the slave I/O thread is running, 0 otherwise.
    """
    statu_name="Rpl_semi_sync_slave_status"

class SlaveIORunning(ShowSlave):
    """
    1 --> yes
    0 --> 其它可能的情况
    """
    show_slave_name="Slave_IO_Running"
    
    def _get_value(self):
        value = super()._get_value()
        if value == 'this node is master':
            return -2 # 直接返回 -2
        if value.upper() == 'YES':
            self._value = 1
            return self._value
        else:
            self._value = 0
            return self._value

class SlaveSQLRunning(ShowSlave):
    """
    1 --> yes
    0 --> 其它可能的情况
    """
    show_slave_name="Slave_SQL_Running"

    def _get_value(self):
        value = super()._get_value()
        if value == 'this node is master':
            return -2 # 直接返回 -2 
        if value.upper() == 'YES':
            self._value = 1
            return self._value
        else:
            self._value = 0
            return self._value

class SecondsBehindMaster(ShowSlave):
    """
    """
    show_slave_name="Seconds_Behind_Master"

    def _get_value(self):
        value = super()._get_value()
        if value == 'this node is master':
            return -2 # 直接返回 -2 
        else:
            try:
                self._value = int(value)
            except Exception as e:
                return -1
            else:
                return self._value 



from .base import ShowMaster

class BinlogFile(ShowMaster):
    show_master_name="File"

class BinlogPosition(ShowMaster):
    show_master_name="Position"

class BinlogDoDB(ShowMaster):
    show_master_name="Binlog_Do_DB"

class BinlogIgnoreDB(ShowMaster):
    show_master_name="Binlog_Ignore_DB"




"""
实现若干通用的核心函数
"""
import os
import sys
import time
import mysql
import random
import string
import logging
import argparse

def check_python_version() -> None:
    """
    检测当前的 python 版本是否被支持，只支持 python-3.0.x 以上的环境
    """
    if sys.version_info.major <= 2:
        print("only support python-3.x",file=sys.stderr)
        sys.exit(1)







from distutils.core import setup

olds_scripts = ['bin/mtlsmonitor', 'bin/mtlsbackup', 'bin/mtlslog', 'bin/mtlsanalysis',
                'bin/mtlshttp', 'bin/mtlsbigfiles', 'bin/mtlsdeleterows']

news_scripts = ['bin/mtls-big-files', 'bin/mtls-delete-rows', 'bin/mtls-file-truncate',
                'bin/mtls-http', 'bin/mtls-log', 'bin/mtls-monitor', 'bin/mtls-backup',
                'bin/mtls-perf-bench', 'bin/mtls-kill-all-conections', 'bin/mtls-sql-distribution',
                'bin/mtls-file-stat', 'bin/mtls-expired-tables', 'bin/mtls-random-passwd',
                'bin/mtsl-rows-diff']

scripts = olds_scripts + news_scripts

setup(name='mysqltools-python',
      version='2.20.09.05',
      scripts=scripts,
      packages=['mtls', 'mtls.kits'],
      maintainer='Neeky',
      maintainer_email='neeky@live.com',
      url='https://github.com/Neeky/mysqltools-python',
      )



# -*- coding:utf8 -*-
"""
定义基类:
    ConnectorBase 代表一个到Mysql数据库的连接
    VariableBase  代表一个查询global variable的连接
    StatuBase     代表一个查询global statu   的连接
"""

__all__ = ['ConnectorBase','VariableBase','StatuBase','PsBase','ShowSlave']

import mysql.connector
import logging
import subprocess
import socket
import json

class Discovery(object):
    def __init__(self,value):
        self._value = value

    @property
    def value(self):
        return json.dumps(self._value)


def is_mysql_port(port):
    """传入一个端口、如果这个端口是MySQL协议用的，那么就返回这个端口，如果不是就返回None
    """
    client_socket = None
    try:
        client_socket = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
        client_socket.connect(('127.0.0.1',port))
        client_socket.settimeout(0.1)
        #MySQL协议下是由Server端先发送握手信息到client的
        message = client_socket.recv(1024)
        message = message.decode('latin-1').lower()
        if 'password' in message:
            return port
    except Exception as e:
        return None
    finally:
        client_socket.close()

def mysql_discovery(*args,**kwargs):
    """查找主机上的MySQL服务并返回它们的监听的port  {"data": [{"{#MYSQLPORT}": 3306}]}
    """
    output = subprocess.check_output(['netstat','-ltn'])
    output = output.decode('latin-1').split('\n')[2:-1]
    result = {"data":[]}
    ports = []
    try:
        for line in output:
            _,_,_,host_and_port,*_ = line.split()
            index = host_and_port.rindex(":")
            port = int(host_and_port[index+1:])
            #只有在这个端口是被MySQL占用的情况下才返回
            if is_mysql_port(port):
                result["data"].append({"{#MYSQLPORT}":port})
    except Exception as e:
        print(e)
        exit()
    return Discovery(value=result)

def disk_discovery(*args,**kwargs):
    result = {"data":[]}
    message = subprocess.check_output(['lsblk',])
    message = message.decode('latin-1')
    for line in message.split('\n'):
        if 'disk' in line:
            disk_name,*_ = line.split()
            result['data'].append({"{#DISKNAME}":disk_name})
    return Discovery(value=result)


class ConnectorBase(object):
    """ConnectorBase代表一个与数据库之间的连接
    ConnectorBase实例的.cursor属性代表着一个连接到数据库的cursor对象
    """
    user='mtsuser'
    password='mts10352'
    host='127.0.0.1'
    port=3306
    _cnx=None
    _cursor=None
    _logger=None

    def __init__(self,host='127.0.0.1',port=3306,user='mtsuser',password='mts10352',database='information_schema',*args,**kws):
        self.host=host
        self.port=port
        self.user=user
        self.password=password
        self.database=database
        self._cnx=None
        self._cursor=None
        self._logger=None
        

    @property
    def cursor(self):
        """返回cursor对象、cursor对象可以完成对数据库的操作
        """
        if self._cursor != None:
            return self._cursor
        else:
            try:
                self._cnx=mysql.connector.connect(user=self.user,password=self.password,host=self.host,port=self.port,database=self.database)
                self._cursor=self._cnx.cursor()
                return self._cursor
            except Exception as e:
                error_message=str(e)
                self.logger.info(error_message)
                self.logger.info("exit")
                self.close()
                exit()
    
    @property
    def logger(self):
        """返回logger对象方便日志的输出
        """
        if self._logger != None:
            return self._logger
        else:
            self._logger=logging.getLogger("mts.base.{0}".format(self.__class__))
            stream_handler=logging.StreamHandler()
            formater=logging.Formatter("%(asctime)-24s %(levelname)-8s %(name)-24s : %(message)s")
            stream_handler.setFormatter(formater)
            self._logger.addHandler(stream_handler)
            self._logger.setLevel(logging.DEBUG)
            return self._logger

    def __str__(self):
        """自定义ConnectorBase对象的字符表示
        """
        obj_str="{0.__class__} instance (host={0.host},port={0.port},user={0.user},password={0.password} )".format(self)
        return obj_str

    def __del__(self):
        """资源回收
        """
        if self._cnx != None:
            self._cnx.close()
    
    def close(self):
        if self._cnx != None:
            self._cnx.close()
        

class VariableBase(ConnectorBase):
    """定义一个用于查询variable的类，类字符variable_name用于指定variable的名字，variable_type用于指定variable对应值的类型
    """
    variable_name=None
    variable_type="string"
    _variable_types=("string","byte","intger","percent","bool")
    _value=None

    def __init__(self,host='127.0.0.1',port=3306,user='mtsuser',password='mts10352',database='information_schema',
                 variable_name=None,variable_type="string",*args,**kws):
        super(VariableBase,self).__init__(host,port,user,password)
        if variable_name != None:
            self.variable_name=variable_name
            self.variable_type=variable_type

    
    def _get_value(self):
        try:
            self.cursor.execute("select @@{0} ;".format(self.variable_name))
            tmp_value=self.cursor.fetchone()
            if tmp_value != None and len(tmp_value)==1:
                return tmp_value[0]
            else:
                self.logger.info("variable {0} has a bad value {1}".format(self.variable_name,tmp_value))
                self.close()
                exit()
        except Exception as e:
                errore_message=str(e)
                self.logger.info(errore_message)
                self.logger.info("exit")
                self.close()
                exit()            

    
    @property
    def value(self):
        #format_mapper={'string':self.format_string_value,
        #               'byte'  :self.format_byte_value,
        #               'intger':self.format_intger_value,
        #               'bool'  :self.format_bool_value,
        #}
        if self._value == None:
            self._value=self._get_value()
        return self._value
        #return format_mapper[self.variable_type](self._value)

    #@property
    #def original_value(self):
    #    return self._get_value()

        
class StatuBase(ConnectorBase):
    statu_name="uptime"
    statu_type="intger"
    _statu_types=("string","byte","intger","percent","bool")
    _value=None

    def __init__(self,host='127.0.0.1',port=3306,user='mtsuser',password='mts10352',
                 statu_name=None,statu_type="intger",*args,**kw):
        super(StatuBase,self).__init__(host,port,user,password)
        if statu_name != None:
            self.statu_name=statu_name
            self.statu_type=statu_type
        self._value=None

    #def format_byte_value(self,raw_value):
    #    """
    #    由于statu 是由show global status like 'xxx' 得到的，所以它返回的是str,对于字节类型的statu,转换一下才行
    #    """
    #    return super(StatuBase,self).format_byte_value(int(self._value))

    def _get_value(self):
        if self._value != None:
            return self._value
        else:
            try:
                self.cursor.execute("show global status like '{0}' ;".format(self.statu_name))
                name_and_value=self.cursor.fetchone()
                if name_and_value == None:
                    self.logger.info("get a None value for statu {0} ".format(self.statu_name))
                    self.close()
                    exit()
                name,value=name_and_value
                self._value=value
                return self._value
            except Exception as e:
                error_message=str(e)
                self.logger.info(error_message)
                self.close()
                exit()

    @property
    def value(self):
        if self._value == None:
            self._value = self._get_value()
        return self._value
        #format_mapper={'string':self.format_string_value,
        #               'intger':self.format_intger_value,
        #               'byte'  :self.format_byte_value,}
        #return format_mapper[self.statu_type](self._get_value())

    #@property
    #def original_value(self):
    #    return self._get_value()
        

class PsBase(ConnectorBase):
    """
    所有与performance_schema操作相关的基类
    """


class ShowSlave(ConnectorBase):
    """通过show slave status 提取信息
    """
    #mysql-8.0.11 版本下('Waiting for master to send event', '127.0.0.1', 'repl', 3307, 60, 'mysql-bin.000001', 151, 'sqlstudio-relay-bin.000002', 357, 'mysql-bin.000001', 'Yes', 'Yes', '', '', '', '', '', '', 0, '', 0, 151, 561, 'None', '', 0, 'No', '', '', '', '', '', 0, 'No', 0, '', 0, '', '', 375, '2c9732e2-8740-11e8-9514-000c29cb87a3', 'mysql.slave_master_info', 0, None, 'Slave has read all relay log; waiting for more updates', 86400, '', '', '', '', '', '', '8e64b57f-83eb-11e8-be2f-000c29cb87a3:1', 1, '', '', '', '', 0)
    show_slave_name=None
    dimensions ={
        'Slave_IO_State':0,
        'Master_Host':1,
        'Master_User':2,
        'Master_Port':3,
        'Connect_Retry':4,
        'Master_Log_File':5,
        'Read_Master_Log_Pos':6,
        'Relay_Log_File':7,
        'Relay_Log_Pos':8,
        'Relay_Master_Log_File':9,
        'Slave_IO_Running':10,
        'Slave_SQL_Running':11,
        'Replicate_Do_DB':12,
        'Replicate_Ignore_DB':13,
        'Replicate_Do_Table':14,
        'Replicate_Ignore_Table':15,
        'Replicate_Wild_Do_Table':16,
        'Replicate_Wild_Ignore_Table':17,
        'Last_Errno':18,
        'Last_Error':19,
        'Skip_Counter':20,
        'Exec_Master_Log_Pos':21,
        'Relay_Log_Space':22,
        'Until_Condition':23,
        'Until_Log_File':24,
        'Until_Log_Pos':25,
        'Master_SSL_Allowed':26,
        'Master_SSL_CA_File':27,
        'Master_SSL_CA_Path':28,
        'Master_SSL_Cert':29,
        'Master_SSL_Cipher':30,
        'Master_SSL_Key':31,
        'Seconds_Behind_Master':32,
        'Master_SSL_Verify_Server_Cert':33,
        'Last_IO_Errno':34,
        'Last_IO_Error':35,
        'Last_SQL_Errno':36,
        'Last_SQL_Error':37,
        'Replicate_Ignore_Server_Ids':38,
        'Master_Server_Id':39,
        'Master_UUID':40,
        'Master_Info_File':41,
        'SQL_Delay':42,
        'SQL_Remaining_Delay':43,
        'Slave_SQL_Running_State':44,
        'Master_Retry_Count':45,
        'Master_Bind':46,
        'Last_IO_Error_Timestamp':47,
        'Last_SQL_Error_Timestamp':48,
        'Master_SSL_Crl':49,
        'Master_SSL_Crlpath':50,
        'Retrieved_Gtid_Set':51,
        'Executed_Gtid_Set':52,
        'Auto_Position':53,
        'Replicate_Rewrite_DB':54,
        'Channel_Name':55,
        'Master_TLS_Version':56,
        'Master_public_key_path':57,
        'Get_master_public_key':58
    }
    dimensions_55 = {
        'Slave_IO_State': 0,
        'Master_Host': 1,
        'Master_User': 2,
        'Master_Port': 3,
        'Connect_Retry': 4,
        'Master_Log_File': 5,
        'Read_Master_Log_Pos': 6,
        'Relay_Log_File': 7,
        'Relay_Log_Pos': 8,
        'Relay_Master_Log_File': 9,
        'Slave_IO_Running': 10,
        'Slave_SQL_Running': 11,
        'Replicate_Do_DB': 12, 
        'Replicate_Ignore_DB': 13,
        'Replicate_Do_Table': 14,
        'Replicate_Ignore_Table': 15,
        'Replicate_Wild_Do_Table': 16, 
        'Replicate_Wild_Ignore_Table': 17,
        'Last_Errno': 18,
        'Last_Error': 19,
        'Skip_Counter': 20,
        'Exec_Master_Log_Pos': 21,
        'Relay_Log_Space': 22,
        'Until_Condition': 23,
        'Until_Log_File': 24,
        'Until_Log_Pos': 25,
        'Master_SSL_Allowed': 26,
        'Master_SSL_CA_File': 27, 
        'Master_SSL_CA_Path': 28,
        'Master_SSL_Cert': 29, 
        'Master_SSL_Cipher': 30,
        'Master_SSL_Key': 31, 
        'Seconds_Behind_Master': 32,
        'Master_SSL_Verify_Server_Cert': 33,
        'Last_IO_Errno': 34,
        'Last_IO_Error': 35,
        'Last_SQL_Errno': 36,
        'Last_SQL_Error': 37,
        'Replicate_Ignore_Server_Ids': 38,
        'Master_Server_Id': 39,
        'Cur_Exec_Relay_Log_File': 40,
        'Cur_Exec_Relay_Log_Pos': 41,
        'Wanted_Purge_Relay_Log': 42,
        'Purged_Relay_Log': 43
    }

    def __init__(self,host='127.0.0.1',port=3306,user='mtsuser',password='mts10352',*args,**kw):
        super().__init__(host,port,user,password)
        self._value=None

    def _get_value(self):
        if self._value != None:
            return self._value
        else:
            try:
                #适配mysql-5.5.x版本的show slave status;
                self.cursor.execute("select @@version")
                mysql_version,*_ = self.cursor.fetchone()
                if '5.5' in mysql_version:
                    self.dimensions = self.dimensions_55

                self.cursor.execute("show slave status")    
                data = self.cursor.fetchone()
                if data == None:
                    self._value = "this node is master"
                    return self._value
                index = self.dimensions[self.show_slave_name]
                self._value = data[index]
                return self._value
            except Exception as e:
                error_message=str(e)
                self.logger.info(error_message)
                self.close()
                exit()  

    @property
    def value(self):
        if self._value == None:
            self._value = self._get_value()
        return self._value


class ShowMaster(ConnectorBase):
    """分析show master status 的输出
    """
    show_master_name = None
    dimensions = {
        'File':0,
        'Position':1,
        'Binlog_Do_DB':2,
        'Binlog_Ignore_DB':3,
        'Executed_Gtid_Set':4,
    }

    def __init__(self,host='127.0.0.1',port=3306,user='mtsuser',password='mts10352',*args,**kw):
        super().__init__(host,port,user,password)
        self._value=None

    def _get_value(self):
        if self._value != None:
            return self._value
        else:
            try:
                self.cursor.execute("show master status")    
                data = self.cursor.fetchone()
                index = self.dimensions[self.show_master_name]
                self._value = data[index]
                return self._value
            except Exception as e:
                error_message=str(e)
                self.logger.info(error_message)
                self.close()
                exit()  

    @property
    def value(self):
        if self._value == None:
            self._value = self._get_value()
        return self._value  



class InnodbStatus(ConnectorBase):
    """分析show engine innodb status 的输出
    """
    def __init__(self,host='127.0.0.1',port=3306,user='mtsuser',password='mts10352',*args,**kw):
        super().__init__(host,port,user,password)
        self._value=None

    def _get_value(self):
        if self._value != None:
            return self._value
        else:
            try:
                self.cursor.execute("show engine innodb status")    
                data = self.cursor.fetchone()
                self._value = data
                return self._value
            except Exception as e:
                error_message=str(e)
                self.logger.info(error_message)
                self.close()
                exit()  

    @property
    def value(self):
        if self._value == None:
            self._value = self._get_value()[2]
            
        return self._value

class InnodbStatusLog(InnodbStatus):
    """返回指定维度的数据
    """
    dimension = "Log sequence number"

    @property
    def value(self):
        _ = super().value
        for line in self._value.split('\n'):
            if self.dimension in line:
                *_,lsn = line.split(self.dimension)
                return lsn.strip()
        return -1


#-*- coding:utf8 -*-

"""
为show global status 中的statu提供一个与之对应的类，以方便使用
"""

from .base import StatuBase

class AbortedClients(StatuBase):
    """
    The number of connections that were aborted because the client died without closing the connection properly.
    """
    statu_name="Aborted_clients"
    statu_type="intger"

class AbortedConnects(StatuBase):
    """
    The number of failed attempts to connect to the MySQL server
    """
    statu_name="Aborted_connects"
    statu_type="intger"

class BinlogCacheDiskUse(StatuBase):
    """
    The number of transactions that used the temporary binary log cache but that exceeded the value of binlog_cache_size and used a temporary file to store statements from the transaction.
    The number of nontransactional statements that caused the binary log transaction cache to be written to disk is tracked separately in the Binlog_stmt_cache_disk_use status variable.
    """
    statu_name="Binlog_cache_disk_use"
    statu_type="intger"

class BinlogCacheUse(StatuBase):
    """
    The number of transactions that used the binary log cache.
    """
    statu_name="Binlog_cache_use"
    statu_type="intger"

class BinlogStmtCacheDiskUse(StatuBase):
    """
    The number of nontransaction statements that used the binary log statement cache but that exceeded the value of binlog_stmt_cache_size and used a temporary file to store those statements.
    """
    statu_name="Binlog_stmt_cache_disk_use"
    statu_type="intger"

class BinlogStmtCacheUse(StatuBase):
    """
    The number of nontransactional statements that used the binary log statement cache.
    """
    statu_name="Binlog_stmt_cache_use"
    statu_type="intger"

class BytesReceived(StatuBase):
    """
    The number of bytes received from all clients.
    """
    statu_name="Bytes_received"
    statu_type="byte"

class BytesSent(StatuBase):
    """
    The number of bytes sent to all clients.
    """
    statu_name="Bytes_sent"
    statu_type="byte"

class ComBegin(StatuBase):
    statu_name="Com_begin"

class ComCallProcedure(StatuBase):
    statu_name="Com_call_procedure"

class ComChangeMaster(StatuBase):
    statu_name="Com_change_master"

class ComCommit(StatuBase):
    statu_name="Com_commit"

class ComDelete(StatuBase):
    statu_name="Com_delete"

class ComDeleteMulti(StatuBase):
    statu_name="Com_delete_multi"

class ComInsert(StatuBase):
    statu_name="Com_insert"

class ComInsertSelect(StatuBase):
    statu_name="Com_insert_select"

class ComSelect(StatuBase):
    statu_name="Com_select"

class ComCreateTable(StatuBase):
    statu_name="Com_create_table"

class ComDropTable(StatuBase):
    statu_name="Com_drop_table"

class ComRenameTable(StatuBase):
    statu_name="Com_rename_table"

class ComAlterTable(StatuBase):
    statu_name="Com_alter_table"

class ComCreateIndex(StatuBase):
    statu_name="Com_create_index"

class ComDropIndex(StatuBase):
    statu_name="Com_drop_index"

class ComCreateUser(StatuBase):
    statu_name="Com_create_user"

class ComAlterUser(StatuBase):
    statu_name="Com_alter_user"

class ComDropUser(StatuBase):
    statu_name="Com_drop_user"

class ComCreateFunction(StatuBase):
    statu_name="Com_create_function"

class ComAlterFunction(StatuBase):
    statu_name="Com_alter_function"

class ComDropFunction(StatuBase):
    statu_name="Com_drop_function"

class ComCreateDb(StatuBase):
    statu_name="Com_create_db"

class ComAlterDb(StatuBase):
    statu_name="Com_alter_db"

class ComDropDb(StatuBase):
    statu_name="Com_drop_db"

class ComCreateProcedure(StatuBase):
    statu_name="Com_create_procedure"

class ComDropProcedure(StatuBase):
    statu_name="Com_drop_procedure"

class ComAlterProcedure(StatuBase):
    statu_name="Com_alter_procedure"

class ComCreateView(StatuBase):
    statu_name="Com_create_view"

class ComDropView(StatuBase):
    statu_name="Com_drop_view"

class ComCreateRole(StatuBase):
    statu_name="Com_create_role"

class ComDropRole(StatuBase):
    statu_name="Com_drop_role"

class ComAlterUserDefaultRole(StatuBase):
    statu_name="Com_alter_user_default_role"

class ComCreateTrigger(StatuBase):
    statu_name="Com_create_trigger"

class ComDropTrigger(StatuBase):
    statu_name="Com_drop_trigger"

class SelectFullJoin(StatuBase):
    """The number of joins that perform table scans because they do not use indexes. 
    If this value is not 0, you should carefully check the indexes of your tables.
    """
    statu_name="Select_full_join"

class SelectFullRangeJoin(StatuBase):
    """The number of joins that used a range search on a reference table.
    """
    statu_name="Select_full_range_join"

class SelectRange(StatuBase):
    """The number of joins that used ranges on the first table. This is normally not a critical issue even if the value is quite large.
    """
    statu_name="Select_range"

class SelectRangeCheck(StatuBase):
    """The number of joins without keys that check for key usage after each row. 
    If this is not 0, you should carefully check the indexes of your tables.
    """
    statu_name="Select_range_check"

class SelectScan(StatuBase):
    """The number of joins that did a full scan of the first table.
    """
    statu_name="Select_scan"

class ComUpdate(StatuBase):
    statu_name="Com_update"

class ComUpdateMulti(StatuBase):
    statu_name="Com_update_multi"

class Connections(StatuBase):
    """
    通过mysql-connector每获取一次这个值，这个值都会加1，这个返回与命令行执行得到的结果不同。
    """
    statu_name="Connections"
    statu_type="intger"

class CreatedTmpDiskTables(StatuBase):
    """
    The number of internal on-disk temporary tables created by the server while executing statements.
    If an internal temporary table is created initially as an in-memory table but becomes too large,
    MySQL automatically converts it to an on-disk table. The maximum size for in-memory temporary tables is the minimum 
    of the tmp_table_size and max_heap_table_size values. If Created_tmp_disk_tables is large, you may want to increase 
    the tmp_table_size or max_heap_table_size value to lessen the likelihood that internal temporary tables in memory will be 
    converted to on-disk tables.You can compare the number of internal on-disk temporary tables created 
    to the total number of internal temporary tables created by comparing the values of the Created_tmp_disk_tables and Created_tmp_tables variables.
    """
    statu_name="Created_tmp_disk_tables"

class CreatedTmpFiles(StatuBase):
    statu_name="Created_tmp_files"

class CreatedTmpTables(StatuBase):
    """
    The number of internal temporary tables created by the server while executing statements.
    You can compare the number of internal on-disk temporary tables created to the total number of internal temporary tables created by comparing the values of the Created_tmp_disk_tables and Created_tmp_tables variables.
    See also Section 8.4.4, “Internal Temporary Table Use in MySQL”.
    Each invocation of the SHOW STATUS statement uses an internal temporary table and increments
    the global Created_tmp_tables value.
    """
    statu_name="Created_tmp_tables"

class InnodbBufferPoolDumpStatus(StatuBase):
    statu_name="Innodb_buffer_pool_dump_status"
    statu_type="string"

class InnodbBufferPoolLoadStatus(StatuBase):
    statu_name="Innodb_buffer_pool_load_status"
    statu_type="string"

class InnodbBufferPoolResizeStatus(StatuBase):
    statu_name="Innodb_buffer_pool_resize_status"
    statu_type="string"

class InnodbBufferPoolBytesData(StatuBase):
    """
    The total number of bytes in the InnoDB buffer pool containing data. 
    The number includes both dirty and clean pages. 
    For more accurate memory usage calculations than with Innodb_buffer_pool_pages_data, 
    when compressed tables cause the buffer pool to hold pages of different sizes.
    """
    statu_name="Innodb_buffer_pool_bytes_data"
    statu_type="byte"

class InnodbBufferPoolPagesData(StatuBase):
    """
    The number of pages in the InnoDB buffer pool containing data. 
    The number includes both dirty and clean pages. When using compressed tables, 
    the reported Innodb_buffer_pool_pages_data value may be larger than Innodb_buffer_pool_pages_total (Bug #59550).
    """
    statu_name="Innodb_buffer_pool_pages_data"

class InnodbBufferPoolPagesDirty(StatuBase):
    """
    The total current number of bytes held in dirty pages in the InnoDB buffer pool. 
    For more accurate memory usage calculations than with Innodb_buffer_pool_pages_dirty, 
    when compressed tables cause the buffer pool to hold pages of different sizes.
    """
    statu_name="Innodb_buffer_pool_pages_dirty"

class InnodbBufferPoolBytesDirty(StatuBase):
    """
    The current number of dirty pages in the InnoDB buffer pool.
    """
    statu_name="Innodb_buffer_pool_bytes_dirty"
    statu_type="byte"

class InnodbBufferPoolPagesFlushed(StatuBase):
    """
    The number of requests to flush pages from the InnoDB buffer pool.
    """
    statu_name="Innodb_buffer_pool_pages_flushed"

class InnodbBufferPoolPagesFree(StatuBase):
    """
    The number of free pages in the InnoDB buffer pool.
    """
    statu_name="Innodb_buffer_pool_pages_free"

class InnodbBufferPoolPagesMisc(StatuBase):
    """
    The number of pages in the InnoDB buffer pool that are busy because they have
    been allocated for administrative overhead, such as row locks or the adaptive hash
    index. This value can also be calculated as 
    Innodb_buffer_pool_pages_total − Innodb_buffer_pool_pages_free − Innodb_buffer_pool_pages_data. 
    When using compressed tables, Innodb_buffer_pool_pages_misc may report an out-of-bounds value (Bug #59550).
    """
    statu_name="Innodb_buffer_pool_pages_misc"

class InnodbBufferPoolPagesTotal(StatuBase):
    """
    The total size of the InnoDB buffer pool, in pages. When using compressed tables, 
    the reported Innodb_buffer_pool_pages_data value may be larger 
    than Innodb_buffer_pool_pages_total (Bug #59550)
    """
    statu_name="Innodb_buffer_pool_pages_total"

class InnodbBufferPoolReadAhead(StatuBase):
    """
    The number of pages read into the InnoDB buffer pool by the read-ahead background thread.
    """
    statu_name="Innodb_buffer_pool_read_ahead"

class InnodbBufferPoolReadAheadEvicted(StatuBase):
    """
    The number of pages read into the InnoDB buffer pool by the read-ahead background thread 
    that were subsequently evicted without having been accessed by queries.
    """
    statu_name="Innodb_buffer_pool_read_ahead_evicted"

class InnodbBufferPoolReadRequests(StatuBase):
    """
    The number of logical read requests.
    """
    statu_name="Innodb_buffer_pool_read_requests"

class InnodbBufferPoolReads(StatuBase):
    """
    The number of logical reads that InnoDB could not satisfy from the buffer pool, 
    and had to read directly from disk.
    """
    statu_name="Innodb_buffer_pool_reads"

class InnodbBufferPoolWaitFree(StatuBase):
    """
    Normally, writes to the InnoDB buffer pool happen in the background. 
    When InnoDB needs to read or create a page and no clean pages are available, 
    InnoDB flushes some dirty pages first and waits for that operation to finish. 
    This counter counts instances of these waits. If innodb_buffer_pool_size has been set properly, 
    this value should be small.
    """
    statu_name="Innodb_buffer_pool_wait_free"

class InnodbBufferPoolWriteRequests(StatuBase):
    """
    The number of writes done to the InnoDB buffer pool.
    """
    statu_name="Innodb_buffer_pool_write_requests"

class InnodbDataFsyncs(StatuBase):
    """
    The number of fsync() operations so far. The frequency of fsync() calls is influenced by the
    setting of the innodb_flush_method configuration option.
    """
    statu_name="Innodb_data_fsyncs"

class InnodbDataPendingFsyncs(StatuBase):
    """
    The current number of pending fsync() operations. The frequency of fsync() calls is influenced 
    by the setting of the innodb_flush_method configuration option.
    """
    statu_name="Innodb_data_pending_fsyncs"

class InnodbDataPendingReads(StatuBase):
    """
    The current number of pending reads.
    """
    statu_name="Innodb_data_pending_reads"

class InnodbDataPendingWrites(StatuBase):
    """
    The current number of pending writes.
    """
    statu_name="Innodb_data_pending_writes"

class InnodbDataRead(StatuBase):
    """
    The amount of data read since the server was started (in bytes).
    """
    statu_name="Innodb_data_read"
    statu_type="byte"

class InnodbDataReads(StatuBase):
    """
    The total number of data reads (OS file reads).
    """
    statu_name="Innodb_data_reads"

class InnodbDataWrites(StatuBase):
    """
    The total number of data writes.
    """
    statu_name="Innodb_data_writes"

class InnodbDataWritten(StatuBase):
    """
    The amount of data written so far, in bytes.
    """
    statu_name="Innodb_data_written"
    statu_type="byte"

class InnodbDblwrPagesWritten(StatuBase):
    """
    The number of pages that have been written to the doublewrite buffer. See Section 14.12.1, “InnoDB Disk I/O”.
    """
    statu_name="Innodb_dblwr_pages_written"

class InnodbDblwrWrites(StatuBase):
    """
    The number of doublewrite operations that have been performed. See Section 14.12.1, 
    “InnoDB DiskI/O”.
    """
    statu_name="Innodb_dblwr_writes"

class InnodbLogWaits(StatuBase):
    """
    The number of times that the log buffer was too small and a wait was required for it to be flushed before continuing.
    """
    statu_name="Innodb_log_waits"

class InnodbLogWriteRequests(StatuBase):
    """
    The number of write requests for the InnoDB redo log.
    """
    statu_name="Innodb_log_write_requests"

class InnodbLogWrites(StatuBase):
    """
    The number of physical writes to the InnoDB redo log file.
    """
    statu_name="Innodb_log_writes"

class InnodbOsLogFsyncs(StatuBase):
    """
    The number of fsync() writes done to the InnoDB redo log files.
    """
    statu_name="Innodb_os_log_fsyncs"

class InnodbOsLogPendingFsyncs(StatuBase):
    """
    The number of pending fsync() operations for the InnoDB redo log files.
    """
    statu_name="Innodb_os_log_pending_fsyncs"

class InnodbOsLogPendingWrites(StatuBase):
    """
    The number of pending writes to the InnoDB redo log files.
    """
    statu_name="Innodb_os_log_pending_writes"

class InnodbOsLogWritten(StatuBase):
    """
    The number of bytes written to the InnoDB redo log files.
    """
    statu_name="Innodb_os_log_written"

class InnodbPagesCreated(StatuBase):
    """
    The number of pages created by operations on InnoDB tables.
    """
    statu_name="Innodb_pages_created"

class InnodbPagesRead(StatuBase):
    """
    The number of pages read from the InnoDB buffer pool by operations on InnoDB tables.
    """
    statu_name="Innodb_pages_read"

class InnodbPagesWritten(StatuBase):
    """
    The number of pages written by operations on InnoDB tables.
    """
    statu_name="Innodb_pages_written"

class InnodbRowLockCurrentWaits(StatuBase):
    """
    The number of row locks currently being waited for by operations on InnoDB tables.
    """
    statu_name="Innodb_row_lock_current_waits"

class InnodbRowLockTime(StatuBase):
    """
    The total time spent in acquiring row locks for InnoDB tables, in milliseconds.
    """
    statu_name="Innodb_row_lock_time" 

class InnodbRowLockTimeAvg(StatuBase):
    """
    The average time to acquire a row lock for InnoDB tables, in milliseconds.
    """
    statu_name="Innodb_row_lock_time_avg"

class InnodbRowLockTimeMax(StatuBase):
    """
    The maximum time to acquire a row lock for InnoDB tables, in milliseconds.
    """
    statu_name="Innodb_row_lock_time_max"

class InnodbRowLockWaits(StatuBase):
    """
    The number of times operations on InnoDB tables had to wait for a row lock.
    """
    statu_name="Innodb_row_lock_waits"

class InnodbRowsDeleted(StatuBase):
    """
    The number of rows deleted from InnoDB tables.
    """
    statu_name="Innodb_rows_deleted"

class InnodbRowsInserted(StatuBase):
    """
    The number of rows inserted into InnoDB tables.
    """
    statu_name="Innodb_rows_inserted"

class InnodbRowsRead(StatuBase):
    """
    The number of rows read from InnoDB tables.
    """
    statu_name="Innodb_rows_read"

class InnodbRowsUpdated(StatuBase):
    """
    The number of rows updated in InnoDB tables.
    """
    statu_name="Innodb_rows_updated"

class InnodbAvailableUndoLogs(StatuBase):
    """
    The number of times output from the SHOW ENGINE INNODB STATUS statement has been truncated.
    """
    statu_name="Innodb_available_undo_logs"

class OpenTableDefinitions(StatuBase):
    """
    The number of cached .frm files.
    """
    statu_name="Open_table_definitions"

class OpenTables(StatuBase):
    """
    The number of tables that are open.
    """
    statu_name="Open_tables"

class OpenedTableDefinitions(StatuBase):
    """
    The number of .frm files that have been cached.
    """
    statu_name="Opened_table_definitions"

class OpenedTables(StatuBase):
    """
    The number of tables that have been opened. If Opened_tables is big, your table_open_cache
    value is probably too small.
    """
    statu_name="Opened_Tables"

class QcacheFreeBlocks(StatuBase):
    """
    The number of free memory blocks in the query cache
    """
    statu_name="Qcache_free_blocks"

class QcacheFreeMemory(StatuBase):
    """
    The amount of free memory for the query cache
    """
    statu_name="Qcache_free_memory"

class QcacheHits(StatuBase):
    """
    The number of query cache hits
    """
    statu_name="Qcache_hits"

class QcacheInserts(StatuBase):
    """
    The number of queries added to the query cache.
    """
    statu_name="Qcache_inserts"

class QcacheLowmemPrunes(StatuBase):
    """
    The number of queries that were deleted from the query cache because of low memory.
    """
    statu_name="Qcache_lowmem_prunes"

class QcacheNotCached(StatuBase):
    """
    The number of noncached queries (not cacheable, or not cached due to the query_cache_type setting).
    """
    statu_name="Qcache_not_cached"

class QcacheQueriesInCache(StatuBase):
    """
    The number of queries registered in the query cache.
    """
    statu_name="Qcache_queries_in_cache"

class QcacheTotalBlocks(StatuBase):
    """
    The total number of blocks in the query cache.
    """
    statu_name="Qcache_total_blocks"

class SlowQueries(StatuBase):
    """
    he number of queries that have taken more than long_query_time seconds. 
    This counter increments regardless of whether the slow query log is enabled. For information about that log
    """
    statu_name="Slow_queries"

class TableLocksImmediate(StatuBase):
    """
    The number of times that a request for a table lock could be granted immediately.
    """
    statu_name="Table_locks_immediate"

class TableLocksWaited(StatuBase):
    """The number of times that a request for a table lock could not be granted immediately and a wait was needed. 
    If this is high and you have performance problems, you should first optimize your queries, 
    and then either split your table or tables or use replication.
    """
    statu_name="Table_locks_waited"


class TableOpenCacheOverflows(StatuBase):
    """
    The number of overflows for the open tables cache. This is the number of times, 
    after a table is opened or closed, a cache instance has an unused entry and the size of the instance is larger 
    than table_open_cache / table_open_cache_instances.
    """
    statu_name="Table_open_cache_overflows"

class ThreadsCached(StatuBase):
    """
    The number of threads in the thread cache.
    """
    statu_name="Threads_cached"

class ThreadsConnected(StatuBase):
    """
    The number of currently open connections.
    """
    statu_name="Threads_connected"

class ThreadsCreated(StatuBase):
    """
    The number of threads created to handle connections. If Threads_created is big,
    you may want to increase the thread_cache_size value. The cache miss rate can be calculated 
    as Threads_created/Connections.
    """
    statu_name="Threads_created"

class ThreadsRunning(StatuBase):
    """
    The number of threads that are not sleeping.
    """
    statu_name="Threads_running"

class Uptime(StatuBase):
    """
    The number of seconds that the server has been up.
    """
    statu_name="Uptime"